@@ -876,7 +876,7 @@ misskey_show_icon(const struct json *js, int iuser, const string *userid)
 	if (__predict_true(opt_show_image)) {
 		char filename[PATH_MAX];
 		const char *avatar_url = json_obj_find_cstr(js, iuser, "avatarUrl");
-		if (avatar_url && userid) {
+		if (avatar_url && userid && !opt_force_blurhash) {
 			// URL の FNV1 ハッシュをキャッシュのキーにする。
 			// Misskey の画像 URL は長いのと URL がネストした構造を
 			// しているので単純に一部を切り出して使う方法は無理。
@@ -932,6 +932,16 @@ misskey_show_icon(const struct json *js, int iuser, const string *userid)
 //   "type" : "image/jpeg",
 //   "url" : "...",
 // }
+//
+// --show-image 内の組み合わせ:
+// force-	sensi-	nsfw=
+// blurhash  tive	show
+// ------	------	------
+//	*		Y		n		Blurhash (shade)
+//	Y		n		*		Blurhash (no shade)
+//	Y		Y		Y		Blurhash (no shade)
+//	n		n		*		画像表示
+//	n		Y		Y		画像表示
 static bool
 misskey_show_photo(const struct json *js, int ifile, int index)
 {
@@ -941,12 +951,22 @@ misskey_show_photo(const struct json *js, int ifile, int index)
 	const char *img_url;
 	uint width = 0;
 	uint height = 0;
-	bool shade;
+	bool shade = false;
 	bool shown = false;
 
 	if (opt_show_image) {
 		bool isSensitive = json_obj_find_bool(js, ifile, "isSensitive");
-		if (isSensitive && opt_nsfw != NSFW_SHOW) {
+		if ((!isSensitive || opt_nsfw == NSFW_SHOW) && !opt_force_blurhash) {
+			// 元画像を表示。thumbnailUrl を使う。
+			img_url = json_obj_find_cstr(js, ifile, "thumbnailUrl");
+			if (img_url == NULL || img_url[0] == '\0') {
+				// なければ、ファイルタイプだけでも表示しとく?
+				goto next;
+			}
+			width  = imagesize;
+			height = imagesize;
+		} else {
+			// Blurhash を表示。
 			const char *blurhash = json_obj_find_cstr(js, ifile, "blurhash");
 			if (blurhash == NULL || blurhash[0] == '\0' ||
 				opt_nsfw == NSFW_ALT)
@@ -981,17 +1001,10 @@ misskey_show_photo(const struct json *js, int ifile, int index)
 			}
 			snprintf(urlbuf, sizeof(urlbuf), "blurhash://%s", blurhash);
 			img_url = urlbuf;
-			shade = true;
-		} else {
-			// 元画像を表示。thumbnailUrl を使う。
-			img_url = json_obj_find_cstr(js, ifile, "thumbnailUrl");
-			if (img_url == NULL || img_url[0] == '\0') {
-				// なければ、ファイルタイプだけでも表示しとく?
-				goto next;
+
+			if (isSensitive && opt_nsfw != NSFW_SHOW) {
+				shade = true;
 			}
-			width  = imagesize;
-			height = imagesize;
-			shade = false;
 		}
 		make_cache_filename(img_file, sizeof(img_file), img_url);
 		shown = show_image(img_file, img_url, width, height, shade, index);
@@ -1029,17 +1042,31 @@ misskey_print_filetype(const struct json *js, int ifile, const char *msg)
 
 // 画像 URL からキャッシュファイル名を作成して返す。
 // "file-<color>-<fontheight>-<url>"(.sixel)
+// ただしこの文字列が長すぎる場合は <url> 部分をハッシュに変更。
+// (rsync がファイル名に7文字くらい足したテンポラリファイルを作ろうとするため
+// 元ファイル名が 255 文字ぎりぎりだとエラーになる)
 static void
 make_cache_filename(char *filename, uint bufsize, const char *url)
 {
-	snprintf(filename, bufsize, "file-%s-%u-%s", colorname, fontheight, url);
+	int len;
 
-	// 面倒な文字を置換しておく。
-	for (char *p = filename; *p; p++) {
-		if (strchr(":/()? ", *p)) {
-			*p = '_';
+	len = snprintf(filename, bufsize, "file-%s-%u-%s",
+		colorname, fontheight, url);
+	if (len < 220) {
+		// 面倒な文字を置換しておく。
+		for (char *p = filename; *p; p++) {
+			if (strchr(":/()? ", *p)) {
+				*p = '_';
+			}
 		}
+	} else {
+		// 長い場合はハッシュにする。
+		string *md5 = hash_md5(url);
+		snprintf(filename, bufsize, "file-%s-%u-%s",
+			colorname, fontheight, string_get(md5));
+		string_free(md5);
 	}
+
 }
 
 // 本文を表示用に整形。
@@ -1128,13 +1155,43 @@ misskey_display_text(const struct json *js, int inote, const char *text)
 		} else if (c == '$' && ustring_at(src, pos + 1) == '[') {
 			// MFM タグ開始。
 			int e = pos + 2;
-			// タグは全部無視するのでタグ名をスキップ。
-			for (; e < posend && srcarray[e] != ' '; e++)
-				;
-			// 空白の次から ']' の手前までが本文。
-			mfmtag++;
-			pos = e + 1;
-			continue;
+
+			// タグ名は空白(SP)区切りだろうか。
+			if (unichar_submatch(&srcarray[e], "ruby ")) {
+				e += 5;
+				// 次の空白と閉じ括弧を書き換えて、ここから本文扱いに戻す。
+				// "..$[ruby ルビ るび]..."
+				//           ^ e はイマココ
+				// "..$[ruby ルビ(るび)..."
+
+				// 空白が連続しているかも知れないのでスキップ。
+				for (; ustring_at(src, e) == ' '; e++)
+					;
+				pos = e;
+				// 一つ目の "ルビ" をスキップ。
+				for (; e < posend && srcarray[e] != ' '; e++)
+					;
+				// この区切り空白も連続してるかも知れないので一旦スキップ。
+				for (; ustring_at(src, e) == ' '; e++)
+					;
+				// "るび" の手前の空白を括弧に書き換える。
+				if (e < posend) {
+					ustring_set_at(src, e - 1, '(');
+				}
+				// タグ終端までスキップ。ネストには未対応。
+				for (; e < posend && srcarray[e] != ']'; e++)
+					;
+				ustring_set_at(src, e, ')');
+				continue;
+			} else {
+				// 知らないタグなら、タグを無視してコンテンツのみ表示。
+				for (; e < posend && srcarray[e] != ' '; e++)
+					;
+				// 空白の次から ']' の手前までが本文。
+				mfmtag++;
+				pos = e + 1;
+				continue;
+			}
 
 		} else if (c == ']' && mfmtag > 0) {
 			// MFM タグ終端。
