@@ -41,8 +41,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#define SAYAKA_VERSION "3.8.4"
-#define SAYAKA_RELDATE "2025/08/10"
+#define SAYAKA_VERSION "3.8.5"
+#define SAYAKA_RELDATE "2025/12/30"
 
 enum {
 	DEFAULT_FONT_WIDTH	= 7,
@@ -97,6 +97,7 @@ int opt_bgtheme;					// -1:自動判別 0:Dark 1:Light
 const char *opt_codeset;			// 出力文字コード (NULL なら UTF-8)
 static uint opt_fontwidth;			// --font 指定の幅   (指定なしなら 0)
 static uint opt_fontheight;			// --font 指定の高さ (指定なしなら 0)
+bool opt_force_blurhash;			// 画像はすべて Blurhash から表示する
 uint opt_nsfw;						// NSFW コンテンツの表示方法
 bool opt_overwrite_cache;			// キャッシュファイルを更新する
 static bool opt_progress;
@@ -118,6 +119,7 @@ enum {
 	OPT_eaw_n,
 	OPT_euc_jp,
 	OPT_font,
+	OPT_force_blurhash,
 	OPT_help,
 	OPT_help_all,
 	OPT_ipv4,
@@ -151,6 +153,7 @@ static const struct option longopts[] = {
 	{ "eaw-n",			required_argument,	NULL,	OPT_eaw_n },
 	{ "euc-jp",			no_argument,		NULL,	OPT_euc_jp },
 	{ "font",			required_argument,	NULL,	OPT_font },
+	{ "force-blurhash",	no_argument,		NULL,	OPT_force_blurhash },
 	{ "help",			no_argument,		NULL,	OPT_help },
 	{ "help-all",		no_argument,		NULL,	OPT_help_all },
 	{ "home",			no_argument,		NULL,	'h' },
@@ -334,6 +337,10 @@ main(int ac, char *av[])
 			break;
 		 }
 
+		 case OPT_force_blurhash:
+			opt_force_blurhash = true;
+			break;
+
 		 case 'h':
 			cmd = CMD_STREAM;
 			is_home = true;
@@ -522,8 +529,8 @@ version(void)
 static void
 usage(void)
 {
-	fprintf(stderr, "usage: %s <command> [<options...>]\n", progname);
-	fprintf(stderr,
+	printf("usage: %s <command> [<options...>]\n", progname);
+	printf(
 " <command>\n"
 "  -h,--home           : Home timeline mode (needs --server and --token)\n"
 "  -l,--local          : Local timeline mode (needs --server)\n"
@@ -543,8 +550,8 @@ usage(void)
 static void
 help_all(void)
 {
-	fprintf(stderr, "usage: %s <command> [<options>...]\n", progname);
-	fprintf(stderr,
+	printf("usage: %s <command> [<options>...]\n", progname);
+	printf(
 " <command>\n"
 "  -h,--home              : Home timeline mode (needs --server and --token)\n"
 "  -l,--local             : Local timeline mode (needs --server)\n"
@@ -555,7 +562,7 @@ help_all(void)
 "     16       : Fixed ANSI compatible 16 colors\n"
 "     8        : Fixed RGB 8 colors\n"
 "     2        : Monochrome image, but use text bold sequence\n"
-"     1        : Monochrome image, and disable any text color sequences\n"
+"     1        : Monochrome image, and disable all text color sequences\n"
 "     gray[<n>]: (2..256) shades of grayscale. If <n> is omitted, 256 is used\n"
 "                'gray2' is a synonym for '2'\n"
 "  --ciphers=<ciphers>    : \"RSA\" can only be specified\n"
@@ -564,6 +571,7 @@ help_all(void)
 "  --eaw-n=<1|2>          : Width of Unicode EAW Neutral char   (defualt:1)\n"
 "  --euc-jp / --jis       : Set output charset\n"
 "  --font=<W>x<H>         : Set font size (Normally autodetected)\n"
+"  --force-blurhash       : Show blurhash image instead of actual image\n"
 "  --help-all             : This help\n"
 "  --ipv4 / --ipv6        : Connect only IPv4/v6 for both stream and images\n"
 "  --mathalpha            : Use alternate character for some MathAlpha chars\n"
