@@ -40,8 +40,8 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 
-#define SIXELV_VERSION "3.8.4"
-#define SIXELV_RELDATE "2025/08/10"
+#define SIXELV_VERSION "3.8.5"
+#define SIXELV_RELDATE "2025/12/30"
 
 typedef enum {
 	OUTPUT_FORMAT_SIXEL,
@@ -67,6 +67,7 @@ static bool ignore_error;			// true ならエラーでも次ファイルを処
 static FILE *ofp;					// 出力中のストリーム
 static bool opt_blurhash_nearest;	// Blurhash を最近傍補間する
 static ResizeAxis opt_resize_axis;
+static bool opt_no_progressive;
 static uint opt_width;
 static uint opt_height;
 static int  opt_page;
@@ -99,6 +100,7 @@ enum {
 	OPT_ipv4,
 	OPT_ipv6,
 	OPT_list,
+	OPT_no_progressive,
 	OPT_output_format,
 	OPT_profile,
 	OPT_resize_axis,
@@ -127,6 +129,7 @@ static const struct option longopts[] = {
 	{ "ipv4",			no_argument,		NULL,	OPT_ipv4 },
 	{ "ipv6",			no_argument,		NULL,	OPT_ipv6 },
 	{ "list",			no_argument,		NULL,	OPT_list },
+	{ "no-progressive",	no_argument,		NULL,	OPT_no_progressive },
 	{ "output-format",	required_argument,	NULL,	'O' },
 	{ "page",			required_argument,	NULL,	'p' },
 	{ "profile",		no_argument,		NULL,	OPT_profile },
@@ -353,6 +356,10 @@ main(int ac, char *av[])
 			}
 			break;
 
+		 case OPT_no_progressive:
+			opt_no_progressive = true;
+			break;
+
 		 case OPT_sixel_or:
 			imageopt.output_ormode = true;
 			break;
@@ -469,9 +476,8 @@ list(void)
 static void
 usage(void)
 {
-	fprintf(stderr,
-		"usage: %s [<options...>] [-|<file|url...>]\n", progname);
-	fprintf(stderr,
+	printf("usage: %s [<options...>] [-|<file|url...>]\n", progname);
+	printf(
 "  -c <color>      : Color mode. 2, 8, 16, 256, gray[2..256] (default:256)\n"
 "                    (See --help-all for more details)\n"
 "  -w <width>      : Resize width to <width> pixel\n"
@@ -499,16 +505,15 @@ usage(void)
 		"", // 偶数パディング
 	};
 	for (uint i = 0; i < countof(opts) / 2; i++) {
-		fprintf(stderr, "  %-39s%s\n", opts[i * 2], opts[i * 2 + 1]);
+		printf("  %-39s%s\n", opts[i * 2], opts[i * 2 + 1]);
 	}
 }
 
 static void
 help_all(void)
 {
-	fprintf(stderr,
-		"usage: %s [<options...>] [-|<file|url...>]\n", progname);
-	fprintf(stderr,
+	printf("usage: %s [<options...>] [-|<file|url...>]\n", progname);
+	printf(
 "  -c,--color=<colormode> : Set color mode (default:256)\n"
 "     256          : A synonym for 'adaptive256'\n"
 "     adaptive[<n>]: Adaptive palette (n=8..256). If <n> is omitted, 256 is used\n"
@@ -546,6 +551,7 @@ help_all(void)
 "  -O,--output-format=<fmt> : ascii, bmp or sixel (default:sixel)\n"
 "  -o <filename>          : Output filename, '-' means stdout (default:-)\n"
 "  -p,--page=<page>       : Specify the page(frame). (animated GIF/WebP only)\n"
+"  --no-progressive       : Don't use progressive data (jxl only)\n"
 "  --sixel-or             : Output SIXEL by OR-mode\n"
 "  --sixel-transbg        : Make SIXEL background transparent\n"
 "  --suppress-palette     : Suppress output of SIXEL palette definition\n"
@@ -655,6 +661,7 @@ do_file(const char *infile)
 		hint.width  = opt_width;
 		hint.height = opt_height;
 		hint.page   = opt_page;
+		hint.progressive = !opt_no_progressive;
 		srcimg = image_read(pstream, loader_idx, &hint, diag_image);
 		if (srcimg) {
 			// 得られた画像サイズと引数指定から、いい感じにサイズを決定。
@@ -704,7 +711,7 @@ do_file(const char *infile)
 
 	PROF(&reduct_end);
 
-	if (GET_COLOR_MODE(imageopt.color) == COLOR_MODE_ADAPTIVE) {
+	if (IS_COLOR_MODE_ADAPTIVE(imageopt.color)) {
 		Debug(diag_image, "AdaptivePalette InputColors=%u%s OutputColors=%u",
 			srcimg->palette_count,
 			(srcimg->palette_count > 256 ? "/32768" : ""),
